#include "LCPmanzini.hpp"

LCPmanzini::LCPmanzini(int* SA, unsigned char* t, int l) {
	this->SA = SA;
	this->text = t;
	this->length = l;
}

int* LCPmanzini::construct() {
	int* LCP;
	int k, j;
	try {
		LCP = new int[length];
	}
	catch(bad_alloc) {
		cerr << "Not enough memory for allocating lcp... exit\n";
		return NULL;
	}

	// init count array:
	int count[256];
	memset(count, 0, sizeof(int) * 256);
	for (int i = 0; i < length; i++) count[(int) text[i]]++;
	k = count[0]; // count of i-1 in the following loop
	count[0] = 0;
	for (int i = 1; i < 256; i++) {
		j = count[i];
		count[i] = count[i-1] + k;
		k = j;
	}

	// calculate the RankNext-information
	char c;
	j = ++count[text[length-1]];
	LCP[j-1] = 0;
	for (int i = 0; i < length; i++) {
		if (SA[i] == 0)
			k = i;
		else {
			c = text[SA[i] - 1];
			j = ++count[c];
			LCP[j-1] = i;
		}
	}

	// now calculate the LCP-array
	int h = 0, nextk;
	unsigned  char *p1, *p0;
	for (int i = 0; i < length; i++) {
		nextk = LCP[k];
		if (k == 0)
			LCP[k] = -1;
		else {
			j = SA[k-1];
			p1 = text + i + h;
			p0 = text + j + h;
			while(*p0++ == *p1++) h++;
			LCP[k] = h;
		}
		if(h > 0) h--;
		k = nextk;
	}

	return LCP;
}

int* LCPmanzini::constructDB() {
	int* LCP;
	int k, j;
	try {
		LCP = new int[length+1]; // +1 for "-1" at the end
	}
	catch(bad_alloc) {
		cerr << "Not enough memory for allocating lcp... exit\n";
		return NULL;
	}

	// init count array:
	int count[256];
	memset(count, 0, sizeof(int) * 256);
	for (int i = 0; i < 256; i++) count[i] = 0;
	for (int i = 0; i < length; i++) count[(int) text[i]]++;
	k = count[0]; // count of i-1 in the following loop
	count[0] = 0;
	for (int i = 1; i < 256; i++) {
		j = count[i];
		count[i] = count[i-1] + k;
		k = j;
	}

	// calculate the RankNext-information
	char c;
	j = ++count[text[length-1]];
	LCP[j-1] = 0;
	for (int i = 0; i < length; i++) {
		if (SA[i] == 0)
			k = i;
		else {
			c = text[SA[i] - 1];
			j = ++count[c];
			LCP[j-1] = i;
		}
	}

	// now calculate the LCP-array
	int h = 0, nextk;
	unsigned  char *p1, *p0;
	for (int i = 0; i < length; i++) {
		nextk = LCP[k];
		if (k == 0)
			LCP[k] = -1;
		else {
			j = SA[k-1];
			p1 = text + i + h;
			p0 = text + j + h;
			// the following line is the only 'real' difference to construct()
			while(*p0 != '\1' && *p1 != '\1' && *p0++ == *p1++) h++;
			LCP[k] = h;
		}
		if(h > 0) h--;
		k = nextk;
	}

	LCP[length] = -1; // acts as a stop-marker at the end
	return LCP;
}
