#ifndef _SAfor2DBnaive_hpp_
#define _SAfor2DBnaive_hpp_

// constants for readable DB-formats:
#define BUILTIN 1
#define FASTA 2
#define FASTA_MAXLINELENGTH 256

#include "constructSA.hpp"
#include "manzini.hpp"
#include <stdlib.h>
#include <iostream>
#include <stdio.h>
#include <sys/stat.h>
#include <time.h>
#include <fstream>
using namespace std;


class SAfor2DBnaive {
public:
  // construct SA with Manzini's method
  SAfor2DBnaive(char*, char*, int);

  // construct SA with given method
  SAfor2DBnaive(char*, char*, int, ConstructSA*);

  // destructor
  virtual ~SAfor2DBnaive();

  // return the index of database entry which occurs in SA at pos. i
  // Currently implemented in time O(log length) instead of O(1)
  // which saves 1 array of size length.
  // DO NOT CALL with Position of 0-byte (i.e., with i=0)
  virtual int index(int);

  // the suffix array (for t)
  int* SA;

  // the text (contains both databases)
  unsigned char* t;

  // number of DB entries in database 1
  int anz1;

  // number of DB entries in database 2
  int anz2;

  // number of DB entries in both DBs (sum of anz1 and anz2)
  int anz;

  // total length of DB (i.e., the SA has length+1 entries (0-byte))
  int length;

  // position in t where second database starts
  int middle;

private:
  // method for constructing the SA (reads complete DB)
  int* construct(ConstructSA*, char*, char*, int, int = 0);

  // like fgets, but return number of chars
  int readln(char*, int, FILE*);

  // saves borders of DB-entries
  int* bound;
};

#endif
